﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace 截图示例
{
    public partial class GetScreenForm : Form
    {
        /// <summary>
        /// 亮图（原图）
        /// </summary>
        public Bitmap bitmap { get; set; }

        /// <summary>
        /// 暗图
        /// </summary>
        public Bitmap bitmap2 { get; set; }

        /// <summary>
        /// 屏幕的宽
        /// </summary>
        public int W { get; set; }
        /// <summary>
        /// 屏幕的高
        /// </summary>
        public int H { get; set; }
        /// <summary>
        /// 适用于高DPI的宽度
        /// </summary>
        public int W2 { get; set; }
        /// <summary>
        /// 适用于高DPI的高度
        /// </summary>
        public int H2 { get; set; }

        Graphics g;
        Bitmap cache;
        Graphics gMain;

        /// <summary>
        /// 构造方法
        /// </summary>
        public GetScreenForm(Bitmap bitmap)
        {
            //亮图 （也就是原图）
            this.bitmap = bitmap;
            this.W = bitmap.Width;
            this.H = bitmap.Height;

            //暗图
            this.bitmap2 = new Bitmap(bitmap.Width, bitmap.Height);
            using (Graphics g = Graphics.FromImage(bitmap2))
                g.DrawImage(bitmap, 0, 0);
            ImageHelper.Lighten(bitmap2, -100);
            //求出适用于高DPI的宽和高
            W2 = (int)(bitmap2.Width * PrimaryScreen.ScaleX);
            H2 = (int)(bitmap2.Height * PrimaryScreen.ScaleY);
            //初始化
            InitializeComponent();
            this.Width = (int)(this.W / PrimaryScreen.ScaleX);
            this.Height = (int)(this.H / PrimaryScreen.ScaleY);
            //绘图相关
            cache = new Bitmap(this.W, this.H);
            gMain = this.CreateGraphics();
            g = Graphics.FromImage(cache);
        }

        /// <summary>
        /// 双击关闭
        /// </summary>
        protected override void OnDoubleClick(EventArgs e)
        {
            //获取截图 
            if (SX > int.MinValue && SY > int.MinValue)
            {
                //获取区域
                int x1 = SX, x2 = SX + SW;
                if (x1 > x2) { x2 = x1 + x2; x1 = x2 - x1; x2 = x2 - x1; };
                int y1 = SY, y2 = SY + SH;
                if (y1 > y2) { y2 = y1 + y2; y1 = y2 - y1; y2 = y2 - y1; };
                //截图
                Bitmap bmp = new Bitmap(x2 - x1, y2 - y1);
                Graphics g6 = Graphics.FromImage(bmp);
                g6.DrawImage(bitmap,
                    new Rectangle(0, 0, bmp.Width, bmp.Height),
                    new Rectangle((int)(x1 * PrimaryScreen.ScaleX), (int)(y1 * PrimaryScreen.ScaleY), (int)((x2 - x1) * PrimaryScreen.ScaleX), (int)((y2 - y1) * PrimaryScreen.ScaleY)),
                    GraphicsUnit.Pixel);
                bmp.Save("x.jpg", ImageFormat.Jpeg);
            }
            this.Close();
        }

        private void GetScreenForm_Load(object sender, EventArgs e)
        {

        }

        protected override void OnShown(EventArgs e)
        {
            DrawForm();
        }

        void DrawForm()
        {
            //画暗图
            g.DrawImage(bitmap2,
                new Rectangle(0, 0, W, H),   //目标
                new Rectangle(0, 0, W2, H2), //源
                GraphicsUnit.Pixel);

            //画亮图
            if (SX > int.MinValue && SY > int.MinValue)
            {
                g.DrawImage(bitmap,
                    new Rectangle(SX, SY, SW, SH),
                    new Rectangle((int)(SX * PrimaryScreen.ScaleX), (int)(SY * PrimaryScreen.ScaleY), (int)(SW * PrimaryScreen.ScaleX), (int)(SH * PrimaryScreen.ScaleY)),
                    GraphicsUnit.Pixel);
                //new Rectangle(SX, SY, SW, SH),   //目标
                //new Rectangle(SX, SY, SW, SH),   //源
                //GraphicsUnit.Pixel);
            }

            //翻转
            gMain.DrawImage(cache, 0, 0);
        }

        /// <summary>
        /// 选择的区域
        /// </summary>
        public int SX { get; set; } = int.MinValue;
        public int SY { get; set; } = int.MinValue;
        public int SW { get; set; }
        public int SH { get; set; }

        /// <summary>
        /// 工作类型 0未工作 1画框 2移框
        /// </summary>
        public int WorkType { get; set; }

        /// <summary>
        /// 移动的起点
        /// </summary>
        public int MoveX { get; set; }
        public int MoveY { get; set; }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            //判断是不是点击在框里
            bool inside = false;
            if (SX > int.MinValue && SY > int.MinValue)
            {
                int x1 = SX, x2 = SX + SW;
                if (x1 > x2) { x2 = x1 + x2; x1 = x2 - x1; x2 = x2 - x1; };
                int y1 = SY, y2 = SY + SH;
                if (y1 > y2) { y2 = y1 + y2; y1 = y2 - y1; y2 = y2 - y1; };
                if (e.X > x1 && e.X < x2
                    && e.Y > y1 && e.Y < y2)
                {
                    inside = true;
                }
            }
            if (inside)
            {
                //在框里，则进行移框
                this.MoveX = e.X;
                this.MoveY = e.Y;
                this.WorkType = 2;
                DrawForm();
            }
            else
            {
                //在框外，则重新画框
                this.SX = e.X;
                this.SY = e.Y;
                this.SW = 0;
                this.SH = 0;
                this.WorkType = 1;
                DrawForm();
            }
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                if (this.WorkType == 1)
                {
                    //画框
                    this.SW = e.X - this.SX;
                    this.SH = e.Y - this.SY;
                }
                else
                {
                    //移框
                    this.SX += e.X - this.MoveX;
                    this.SY += e.Y - this.MoveY;
                    this.MoveX = e.X;
                    this.MoveY = e.Y;
                }
                DrawForm();
            }
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            if (this.WorkType == 1)
            {
                this.SW = e.X - this.SX;
                this.SH = e.Y - this.SY;
            }
            this.WorkType = 0;
            DrawForm();
        }
    }
}
